#include <string>
#include <sys/stat.h>
#include "fm_fileinfo.h"
#include "fm_window.h"
#include "fm_keyboard.h"
#include "fm_conio.h"
#include "fm_vfs.h"
#include "sc_file.h"

struct {
  long int count, measure, dirs,executables;
} sinfo;

void ClearInfo() {
  sinfo.count=sinfo.measure=sinfo.dirs=sinfo.executables=0;
}

bool StatScanDir(const string& dirname) {
  tDirContainer dt;
  ScanForFiles(dt,dirname,false);
  for(int i=0; i<dt.Size(); i++) {
    if (IsKeyPressed()) return false;
    ShowRotator();
    if (dt[i].isdir) {
      if (dt[i].name!=".." && dt[i].name!=".") {
        StatScanDir(AddSlash(dirname)+dt[i].name);
        sinfo.dirs++;
      }
    } else {
      string s=dt[i].name;
      sinfo.count++;
      sinfo.measure+=dt[i].size;
      if (dt[i].isexecutable) sinfo.executables++;
    }
  }
  return true;
}

string InfoJPG(const string&);
string InfoGIF(const string&);
string InfoWAV(const string&);


string GetMoreFileInfo(const cDirEntry& de) {
  string ext=GetPartExt(de.fullname);
  if (ext==".JPG" || ext==".JPEG") return InfoJPG(de.fullname);
  if (ext==".GIF") return InfoGIF(de.fullname);
  if (ext==".WAV") return InfoWAV(de.fullname);
  return "none";
}

void ShowDirInfo(const string& dir) {
  ClearInfo();
  cInfoWindow w("DirInfo",cWHITE);
  w.AddLine("Complete name:",chYELLOW);
  w+=dir;
  /*w.AddLine("Time of last modification:",chYELLOW);
  w+=de.GetFileTimeStr();*/
  w.AddLine("Statistical information:",chYELLOW);
  ShowMessage("Scanning...");
    
  if (StatScanDir(dir.c_str())) {
    w+="Number of files    : "+MkStr(sinfo.count,true)+" files";
    w+="Total size         : "+MkStr(sinfo.measure,true)+" bytes";
    w+="Number of subdirs  : "+MkStr(sinfo.dirs,true)+" dirs";
    w+="Number of programs : "+MkStr(sinfo.executables,true)+" executables";
    
  } else {
    w.AddLine("Error scanning!",cRED);
  }
  //WaitKey();
  w.Show();
}

void ShowFileInfo(const cDirEntry& de) {
  if (de.isdir) {
    ShowDirInfo(de.fullname);
  } else {
    cInfoWindow w("FileInfo");
    w.AddLine("Complete name:",chYELLOW);
    w+=de.fullname;

    w.AddLine("Time of last modification:",chYELLOW);
    w+=de.GetFileTimeStr();
    
    w.AddLine("Additional fileinfo:",chYELLOW);
    w+=GetMoreFileInfo(de);
    w.Show();
  }
}

void ShowFileInfo(const tDirContainer& filelist) {
  ClearInfo();
  for (int i=0; i<filelist.Size(); i++) {
    cDirEntry de=filelist.Read(i);
    #ifdef _SYMLINKS_EXIST_
      if (de.islink) continue;
    #endif
    if (de.tagged) {  //exclude specials, ".." etc.
      if (de.isdir) {
/*        string orig=CurrentDir();  //save original dir
        if (SetDir(dir)) {         //goto the dir to scan
          if (!StatScanDir()) {    //Do the scanning
          SetDir(orig);            //Go back to the original
}*/
        StatScanDir(de.fullname.c_str());
        sinfo.dirs++;
      } else {
        sinfo.count++;
        sinfo.measure+=de.size;
        if (de.isexecutable) sinfo.executables++;
      }
    }
  }
  cInfoWindow w("DirInfo",cWHITE);
  w.AddLine("Tagged files",chYELLOW);
  w+="Number of files    : "+MkStr(sinfo.count,true)+" files";
  w+="Total size         : "+MkStr(sinfo.measure,true)+" bytes";
  w+="Number of subdirs  : "+MkStr(sinfo.dirs,true)+" dirs";
  w+="Number of programs : "+MkStr(sinfo.executables,true)+" executables";
  w.Show();
}


string InfoJPG(const string& filename) {
  string temp;
  int sizex,sizey,colors;
  union {
    unsigned char buf[1024];
    char readbuf[1024];
  };
  int read=GetChunkOfFile(filename,1024,readbuf);
  if (read<1) return "unable to get info";
  for (int i=0; i<(read-1); i++) {
    if (buf[i]==0xff && buf[i+1]==0xc0) {
      sizex=(buf[i+5]*256)+buf[i+6];
      sizey=(buf[i+7]*256)+buf[i+8];
      temp=(MkStr(sizex)+"x"+MkStr(sizey)+" ");
      colors=buf[i+9];
      switch (colors) {
        case 1 : temp+=" 8 bit grayscale"; break;
        case 3 : temp+=" 24bit color RGB encoded"; break;
        case 4 : temp+=" 24bit color CMYK encoded"; break;
        default : temp+=" unknown colorcoding";
      }
      return ("JPG-picture: "+temp);
    }
  }
  return "unable to get info";
}


string InfoGIF(const string& filename) {
  union {
    struct {
      char version[6];
      unsigned short sizex, sizey;
      unsigned char packed, background, pixelaspect;
    } gifheader;
    char buf[20];
  };
  int read=GetChunkOfFile(filename,20,buf);
  if (read<1) return "unable to get info";
  gifheader.version[5]='\0';
  string temp=gifheader.version;
  temp=("GIF-picture: "+temp+" "+MkStr(gifheader.sizex)+"x"+MkStr(gifheader.sizey));
  return temp;
}

string InfoWAV(const string& filename) {
  union {
    struct {
      char riff[4];//"RIFF"
      int rifflength;
      char wave[4];//"WAVE"
      char fmt[4]; //"fmt "
      int fmtlength; //length of infoblock (16)
      unsigned short formattag; //?
      unsigned short channels; //1=mono, 2=stereo
      int samplerate; //samples per second
      int samppersec; //channels * samplrate
      unsigned short blockalign; //1=byte
      unsigned short samplesize; //bits pr. sample (8/16)
      char dataheader[4]; //"data"
      int wavelength;//length of PCM data
    } wavheader;
    char buf[128];
  };
  int read=GetChunkOfFile(filename,20,buf);
  if (read<1) return "unable to get info";
  if (wavheader.riff[0]!='R' || wavheader.riff[1]!='I' || wavheader.riff[2]!='F' || wavheader.riff[3]!='F')
    return "unabled to get info";
  //OK, lets assumes its ok now. We could check the other strings, too..
  string temp;
  temp=MkStr(wavheader.samplerate)+" hz.";
  if (wavheader.channels==1) temp+=" mono"; else temp+=" stereo";

  return temp;
}
/*

Var buf : Array[1..SizeOf(WAVHead)] of Char;
    tw : Word;
    ts : String;
    tr : Real;
    tl : LongInt;
    w : ^fmView;
Begin
  New(FF,Init(Cr.Name, ReadMode));
  If FF^.Open Then Begin
    tw:=SizeOf(WAVHead);
    If ff^.BlockRead(buf,tw) Then Begin
      With WAVHead(buf) do Begin
        If Riff = 'RIFF' Then Begin {presumably OK}
{          DrawCenterWindow (40, 8, 'FileInfo');}
          ViewsBG:=LightGray;
          New(w, ShortInit(40, 8, 'FileInfo'));
          W^.WriteCol (1, 1, 'Type    : Wave-File', Yellow);
          W^.JustWriteCol('Name    : ' + cr. Name, Yellow);
          {If Channels=1 Then ts:='8 bit, uncompressed' else ts:=MStr(wFormatTag);}
{          Wr (Black, Cyan, 1, 4, 'Format      : '+ts, 0);}
          W^.JustWrite('');
          W^.JustWrite('SampleRate  : '+MStr(SampleRate)+' hz');
          If Channels=1 Then ts:='Mono' else ts:='Stereo';
          W^.JustWrite('Channels    : '+ts);
          W^.JustWrite('Precision   : '+MStr(SampleSize)+' bit');
          tl:=Round(WaveLength / SampPerSec);
          If tl>60 Then ts:=MStr(tl div 60)+' min '+Mstr(tl mod 60)+' sec' else ts:=Mstr(tl mod 60)+' sec';
          W^.JustWrite ('Playing Time: '+ts);
          WaitKeyWithTime;
          Dispose(w,Done);
        End;
      End;
    End;
  End;
  Dispose(FF,Done(FALSE));
End;*/


